# -*- coding: utf-8 -*-
# mnis.mnislib module

"""
Name: mnislib.py
Author: Oliver Hawkins
About: A library of ad-hoc functions for downloading data from 
the Members Names Information Service (MNIS) at data.parliament.uk. 

A description of the MNIS API can be found here:
http://data.parliament.uk/membersdataplatform/memberquery.aspx
http://data.parliament.uk/membersdataplatform/fixedscope.aspx

OData API links
http://data.parliament.uk/membersdataplatform/open.aspx
http://data.parliament.uk/membersdataplatform/schema/main.html
http://stackoverflow.com/questions/10112376/how-to-consume-odata-service-with-html-javascript

"""

import datetime
import requests
import json
import csv
import mnis.housedata as housedata

# Package Exceptions ---------------------------------------------------------

class Error(Exception):
	
	"""Base class for exceptions in this module."""	
	pass

class ParameterError(Error):
	"""Exception raised for errors with request parameters."""	
	pass

class ElectionIdError(ParameterError):
	"""Exception raised for invalid election ids."""	
	pass

class MembershipError(Error):
	"""Exception raised when membership data from MNIS is nonsensical."""
	pass


# Functions which return a list of members -----------------------------------

def getCurrentCommonsMembers(outputParameters= \
	['Constituencies', 'Parties', 'HouseMemberships']):

	"""Returns all Commons members on the current date."""

	return getCommonsMembersOn(datetime.date.today(), outputParameters)


def getCommonsMembersOn(date, outputParameters= \
	['Constituencies', 'Parties', 'HouseMemberships']):

	"""
	Returns all Commons members on a given date, which must be
	an instance of datetime.date.
	"""
	
	return getCommonsMembersBetween(date, date, outputParameters)


def getCommonsMembersBetween(startDate, endDate, outputParameters= \
	['Constituencies', 'Parties', 'HouseMemberships']):

	"""
	Returns all Commons members between startDate and endDate,
	which must be instances of datetime.date.
	"""

	s = '{0}-{1}-{2}'.format(startDate.year, startDate.month, startDate.day)
	e = '{0}-{1}-{2}'.format(endDate.year, endDate.month, endDate.day)
	urlParameters = 'commonsmemberbetween={0}and{1}'.format(s, e)
	
	return getCommonsMembers(urlParameters, outputParameters)


def getCommonsMembersAtElection(generalElectionId, outputParameters= \
	['Constituencies', 'Parties', 'HouseMemberships']):

	"""
	Returns all Commons members elected at a given general election.
	The MNIS system holds complete data on general elections since 1983. 
	The generalElectionId must be one of the following strings: 1983, 1987, 
	1992, 1997, 2001, 2005, 2010, 2015.
	"""

	validElectionIds = [ \
		'1983', '1987', '1992', '1997', '2001', '2005', '2010', '2015']

	if generalElectionId not in validElectionIds:
		raise ElectionIdError("Invalid id in getCommonsMembersAtElection")

	urlParameters = 'returnedatelection={0}' \
		'%20General%20Election'.format(generalElectionId)

	return getCommonsMembers(urlParameters, outputParameters)


def buildMnisUrl(urlParameters, outputParameters):

	"""
	Builds a valid MNIS URL for the given parameters. The function raises a
	parameter error if more than three output parameters are passed as the
	API only allows up to three output parameters per request.
	"""

	if len(outputParameters) > 3:
		raise ParameterError( \
			"More than three output parameters in buildMnisUrl")

	url = 'http://data.parliament.uk/membersdataplatform/services/' \
		'mnis/members/query/House=Commons|Membership=all|{0}/{1}' \
		.format(urlParameters, '|'.join(outputParameters))

	return url


def getCommonsMembers(urlParameters, outputParameters= \
	['Constituencies', 'Parties', 'HouseMemberships']):

	"""
	Returns all commons members with the given URL paramemters. The 
	"house=Commons" parameter is not necessary as it is provided by 
	buildMnisUrl. The output parameters specify what additional information 
	about MPs the API should return. The API only allows up to three 
	output parameters per request.
	"""

	# Set http request parameters 
	headers = {'content-type': 'application/json'}
	url = buildMnisUrl(urlParameters, outputParameters)

	# Make request
	response = requests.get(url, headers=headers)
	
	# Get response text
	responseText = response.text
	
	# Parse JSON (cutting off the byte order marker at the start)
	members = json.loads(responseText[1:])
	members = members['Members']['Member']

	return members


# Functions that retrieve and process data for members ----------------------

def getIdForMember(member):

	""" 
	Returns the member's unique id as a string.
	"""

	return member['@Member_Id']


def getListNameForMember(member):

	""" 
	Returns the member's list namex, which has the format:
	"surname, (title) firstname", with title being optional.
	"""

	return member['ListAs']


def getGenderForMember(member):

	""" Returns the member's gender as a string: 'F' or 'M'. """

	return member['Gender']


def getDateOfBirthForMember(member):

	""" Returns the member's date of birth as a datetime.date. """

	if isinstance(member['DateOfBirth'], str):
		
		return convertMnisDatetime(member['DateOfBirth'])

	else:

		return ''


def getConstituencyForMember(member, onDate):

	"""
	Returns a member's constituency on a given date. If the member was not an 
	MP on the given date the function returns a string indicating the member 
	was not serving on the specified date. The member must be a member object
	returned by one of the getCommonsMembers functions, and must contain data
	on constituency memberships which is requested with the output parameter 
	for 'Constituencies'. This parameter is one of the defaults for the 
	getCommonsMembers functions. The onDate should be a datetime.date.
	"""

	# Get the data on historic constituency memberships
	constituencyMembership = member['Constituencies']['Constituency']
	constituencyName = 'Not serving on {0}'.format(onDate)

	# Constituency memberships are in a list if there is more than one
	if isinstance(constituencyMembership, list):

		for membership in constituencyMembership:

			if isDateInMembership(membership, onDate):

				constituencyName = membership['Name']
				break

	# Otherwise the membership is accessed directly
	else:
		
		membership = constituencyMembership

		if isDateInMembership(membership, onDate):
			constituencyName = membership['Name']

	return constituencyName


def getPartyForMember(member, onDate):

	"""
	Returns a member's party on a given date. If the member was not an MP on
	the given date the function returns a string indicating the member was not 
	serving on the specified date. The member must be a member object returned 
	by one of the getCommonsMembers functions, and must contain data on party 
	memberships which is requested with the output parameter for 'Parties'. 
	This parameter is one of the defaults for the getCommonsMembers functions. 
	The onDate should be a datetime.date.
	"""

	# Get the data on historic party memberships
	partyMembership = member['Parties']['Party']
	partyName = 'Not serving on {0}'.format(onDate)

	# Party memberships are in a list if there is more than one
	if isinstance(partyMembership, list):

		for membership in partyMembership:

			if isDateInMembership(membership, onDate):

				partyName = membership['Name']
				break

	# Otherwise the membership is accessed directly
	else:
		
		membership = partyMembership

		if isDateInMembership(membership, onDate):
			partyName = membership['Name']

	return partyName


def isDateInMembership(membership, onDate):

	""" Checks whether a date falls within a given membership. """

	# Set the start date to the membership start date
	startDate = convertMnisDatetime(membership['StartDate'])

	# Assume the membership is open and set the end date to today's date
	endDate = datetime.date.today()

	# If the membership is closed set the end date to the membership end date
	if isinstance(membership['EndDate'], str):
		endDate = convertMnisDatetime(membership['EndDate'])	

	return isDateInRange(onDate, startDate, endDate)


def getServiceDataForMember(member, onDate):

	"""
	Returns the date a member first became an MP and the total number of days 
	a member has served up to the given date, excluding periods when the House
	is in dissolution. The member must be a	member object returned by one of 
	the getCommonsMembers functions, and must contain data on House memberships 
	which is requested with the output parameter for 'HouseMemberships'. This 
	parameter is one of the defaults for the getCommonsMembers functions. The 
	onDate should be a datetime.date.
	"""

	# Get Commons memberships to calculate length of service in days
	houseMembership = member['HouseMemberships']['HouseMembership']
	serviceDays = 0
	startDates = []

	# House memberships are in a list if there is more than one
	if isinstance(houseMembership, list):

		for membership in houseMembership:

			if membership['House'] == 'Commons':

				daysService = getMembershipDays(membership, onDate)
				serviceDays += daysService
				startDates.append(convertMnisDatetime(membership['StartDate']))

	# Otherwise the membership is accessed directly
	else:

		if houseMembership['House'] == 'Commons':

			membership = houseMembership
			daysService = getMembershipDays(membership, onDate)
			serviceDays += daysService
			startDates.append(convertMnisDatetime(membership['StartDate']))

	startDates.sort()
	startDate = startDates[0]

	return startDate, serviceDays


def getMembershipDays(membership, onDate, houseDates=housedata.dates):

	"""
	Returns the number of days service in a membership up to the given date,
	excluding periods when the House is in dissolution. The onDate is a
	datetime.date.
	"""

	# Set the start date to the membership start date
	startDate = convertMnisDatetime(membership['StartDate'])

	# Assume the membership is open and set the end date to the onDate
	endDate = onDate

	# If the membership is closed set the end date to the membership end date
	if isinstance(membership['EndDate'], str):
		endDate = convertMnisDatetime(membership['EndDate'])

	# If the membership starts on or after the onDate return zero days
	if startDate >= onDate: return 0

	# If the membership ends after the onDate set it to the onDate
	if endDate > onDate: endDate = onDate

	# If membership ends before it starts raise an error
	if startDate > endDate:
		raise MembershipError("startDate after endDate in getMembershipDays")

	# Set initial service length as the number of days from start to end
	serviceDelta = endDate - startDate
	serviceDays = serviceDelta.days

	# Then remove dissolution periods that fall within the membership
	elections = sorted(houseDates.keys())

	for e in elections:

		election = houseDates[e]['election']
		dissolution = houseDates[e]['dissolution']

		if dissolution >= startDate:

			if dissolution < endDate:

				if election <= endDate:

					# remove entire dissolution and continue
					delta = election - dissolution
					serviceDays -= delta.days
					continue

				else:

					# remove endDate - dissolution
					delta = endDate - dissolution
					serviceDays -= delta.days
					continue
			else:

				# dissolution is after membership so do nothing
				pass

		elif dissolution < startDate:

			if election > startDate:

				if election <= endDate:

					# remove election - startDate
					delta = election - startDate
					serviceDays -= delta.days
					continue

				else:

					# remove whole membership and continue
					delta = endDate - startDate
					serviceDays -= delta.days
					continue

			else:

					# election is before membership so do nothing
					pass

		else:

			# This should not be able to happen
			raise MembershipError("Shouldn't happen in getMembershipDays")

	return serviceDays


# Functions for summarising data on a list of members ------------------------

def getSummaryDataForMembers(members, onDate):

	"""
	Takes a list of members, produces a set of summary data for each member 
	in the list, and returns it as a list of dictionaries. The data returned 
	for a member is determined by the onDate, which should be a datetime.date.
	The data downloaded for each member is:

	- member id
	- listed name
	- constituency
	- party
	- date of birth
	- gender
	- date first became an mp
	- number of days service

	In order to produce this data the members passed to the function must
	contain outputs that are requested with output parameters for:
	
	- Constituencies
	- Parties
	- HouseMemberships

	These are the default output parameters for getCommonsMembers functions.
	"""

	summary = []

	for member in members:

		memberData = {}
		memberData['member_id']	= getIdForMember(member)	
		memberData['list_name'] = getListNameForMember(member)
		memberData['constituency'] = getConstituencyForMember(member, onDate)
		memberData['party'] = getPartyForMember(member, onDate)
		memberData['date_of_birth'] = getDateOfBirthForMember(member)
		memberData['gender'] = getGenderForMember(member)
		memberData['first_start_date'], memberData['days_service'] = \
			getServiceDataForMember(member, onDate)
		summary.append(memberData)

	return summary


def saveSummaryDataForMembers(summaryData, csvName):

	"""
	Takes a list of summary data for each member from getSummaryDataForMembers
	and writes the data to file as a csv with the given filename. The data 
	downloaded for each member is:

	- member id
	- listed name
	- constituency
	- party
	- date of birth
	- gender
	- date first became an mp
	- number of days service

	In order to produce this data the members passed to the function must 
	contain outputs that are requested with output parameters for: 
	
	- Constituencies
	- Parties
	- HouseMemberships

	These are the default output parameters for getCommonsMembers functions.
	"""

	with open(csvName, 'w') as csvFile:
		
		fieldnames = ['member_id', 'list_name', 'constituency', 'party', \
			'date_of_birth', 'gender', 'first_start_date', 'days_service']

		writer = csv.DictWriter(csvFile, fieldnames=fieldnames)
		writer.writeheader()
		
		for summary in summaryData:
			writer.writerow(summary)


def downloadMembers(onDate, csvName):

	"""
	Downloads summary data on members serving in the House of Commons on a 
	given date and saves it as a csv. The onDate should be a datetime.date.
	The csvName is the file into which the data will be saved. The data 
	downloaded for each member is:

	- member id
	- listed name
	- constituency
	- party
	- date of birth
	- gender
	- date first became an mp
	- number of days service

	The data shows information about each members on the given date, so the
	constituency, party, and number of days service is as at the onDate. 
	"""

	members = getCommonsMembersOn(onDate)
	summaryData = getSummaryDataForMembers(members, onDate)
	saveSummaryDataForMembers(summaryData, csvName)


# Utility functions ----------------------------------------------------------

def convertMnisDatetime(mnisDatetime):

	"""
	Takes a string representing a datetime in MNIS data and returns it
	as a datetime.date.
	"""

	mnisDate = mnisDatetime[:10]
	convertedDate = datetime.datetime.strptime(mnisDate, '%Y-%m-%d').date()
	return convertedDate


def isDateInRange(onDate, startDate, endDate):

	"""
	Checks whether a onDate falls between startDate an endDate. The test is 
	inclusive: it passes if onDate is equal to startDate or endDate. All dates
	are datetime.dates. An error is raised if startDate is later than endDate.
	"""

	if startDate > endDate:
		raise MembershipError("startDate later than endDate in isDateInRange")

	if onDate >= startDate and onDate <= endDate:
		return True
	else:
		return False




from mnis.mnislib import \
	getCurrentCommonsMembers, \
	getCommonsMembersOn, \
	getCommonsMembersBetween, \
	getCommonsMembersAtElection, \
	getCommonsMembers, \
	getIdForMember, \
	getListNameForMember, \
	getGenderForMember, \
	getDateOfBirthForMember, \
	getConstituencyForMember, \
	getPartyForMember, \
	getServiceDataForMember, \
	getSummaryDataForMembers, \
	saveSummaryDataForMembers, \
	downloadMembers \


# -*- coding: utf-8 -*-
# mnis.housedata module

"""
Name: housedata.py
Author: Oliver Hawkins
About: This module provides data on the length of Parliaments and 
dissolution periods for use with the mnis package and mnislib module.
Dissolution dates can be found here:
http://researchbriefings.files.parliament.uk/documents/RP15-11/RP15-11.pdf

"""

import datetime

dates = {}

dates['1929'] = {
	'dissolution': datetime.date(1929, 5, 10), 
	'election': datetime.date(1929, 5, 30)
}

dates['1931'] = {
	'dissolution': datetime.date(1931, 10, 7), 
	'election': datetime.date(1931, 10, 27)
}

dates['1935'] = {
	'dissolution': datetime.date(1935, 10, 25), 
	'election': datetime.date(1935, 11, 14)
}

dates['1945'] = {
	'dissolution': datetime.date(1945, 6, 15), 
	'election': datetime.date(1945, 7, 5)
}

dates['1950'] = {
	'dissolution': datetime.date(1950, 2, 3), 
	'election': datetime.date(1950, 2, 23)
}

dates['1951'] = {
	'dissolution': datetime.date(1951, 10, 5), 
	'election': datetime.date(1951, 10, 25)
}

dates['1955'] = {
	'dissolution': datetime.date(1955, 5, 6), 
	'election': datetime.date(1955, 5, 26)
}

dates['1959'] = {
	'dissolution': datetime.date(1959, 9, 18), 
	'election': datetime.date(1959, 10, 8)
}

dates['1964'] = {
	'dissolution': datetime.date(1964, 9, 25), 
	'election': datetime.date(1964, 10, 15)
}

dates['1966'] = {
	'dissolution': datetime.date(1966, 3, 10), 
	'election': datetime.date(1966, 3, 31)
}

dates['1970'] = {
	'dissolution': datetime.date(1970, 5, 29), 
	'election': datetime.date(1970, 6, 18)
}

dates['1974 (Feb)'] = {
	'dissolution': datetime.date(1974, 2, 8), 
	'election': datetime.date(1974, 2, 28)
}

dates['1974 (Oct)'] = {
	'dissolution': datetime.date(1974, 9, 20), 
	'election': datetime.date(1974, 10, 10)
}

dates['1979'] = {
	'dissolution': datetime.date(1979, 4, 7), 
	'election': datetime.date(1979, 5, 3)
}

dates['1983'] = {
	'dissolution': datetime.date(1983, 5, 13), 
	'election': datetime.date(1983, 6, 9)
}

dates['1987'] = {
	'dissolution': datetime.date(1987, 5, 18), 
	'election': datetime.date(1987, 6, 11)
}

dates['1992'] = {
	'dissolution': datetime.date(1992, 3, 16), 
	'election': datetime.date(1992, 4, 9)
}

dates['1997'] = {
	'dissolution': datetime.date(1997, 4, 8), 
	'election': datetime.date(1997, 5, 1)
}

dates['2001'] = {
	'dissolution': datetime.date(2001, 5, 14), 
	'election': datetime.date(2001, 6, 7)
}

dates['2005'] = {
	'dissolution': datetime.date(2005, 4, 11), 
	'election': datetime.date(2005, 5, 5)
}

dates['2010'] = {
	'dissolution': datetime.date(2010, 4, 12), 
	'election': datetime.date(2010, 5, 6)
}

dates['2015'] = {
	'dissolution': datetime.date(2015, 3, 30), 
	'election': datetime.date(2015, 5, 7)
}


from distutils.core import setup
setup(
  name = 'mnis',
  packages = ['mnis'],
  version = '1.0.13',
  description = 'A small library that makes it easy to download data on UK Members of Parliament',
  author = 'Oliver Hawkins',
  author_email = 'oli@olihawkins.com',
  url = 'https://github.com/olihawkins/mnis',
  download_url = 'https://github.com/olihawkins/mnis/tarball/1.0.13',
  keywords = ['MPs', 'Parliament', 'House of Commons'],
  install_requires = ['requests'],
  classifiers = [],
)


# -*- coding: utf-8 -*-
# mnis unit testing module

import unittest
import datetime
import requests
import json
import os
import csv
import mnis.mnislib as mnislib

print("Running tests: Note that the tests require two downloads and can take up to 20 seconds ...")

def downloadMembersForTesting():

	"""Downloads data using a fixed URL for testing against library output."""

	# Set http request parameters for members at GE2015 
	headers = {'content-type': 'application/json'}
	url = 'http://data.parliament.uk/membersdataplatform/' \
		'services/mnis/members/query/House=Commons|Membership=all|' \
		'commonsmemberbetween=2015-05-07and2015-05-07/Constituencies|' \
		'Parties|HouseMemberships'

	# Make request
	response = requests.get(url, headers=headers)
	
	# Get response text
	responseText = response.text
	
	# Parse JSON (cutting off the byte order marker at the start)
	members = json.loads(responseText[1:])
	members = members['Members']['Member']

	# Sort members to ensure they appear in list name alphabetical order
	members.sort(key=lambda member: member['ListAs'])

	return members


# A global module variable used to store downloaded data on members. This data 
# is used for testing functions that work with downloaded data.
correctMembers = downloadMembersForTesting() 


def mockGetCommonsMembers(urlParameters, outputParameters):

	"""
	Mocks getCommonsMembers() and returns the parameters it uses to request 
	data from the MNIS API. This mock is used to test all functions that get 
	Commons members that wrap getCommonsMembers() itself. It simply checks 
	that they construct and pass on the correct parameters given their inputs.
	"""

	return urlParameters, outputParameters


class testGetCurrentCommonsMembers(unittest.TestCase):

	"""
	Tests getCurrentCommonsMembers by mocking getCommonsMembers
	and checking that the parameters passed to it are correct.
	"""

	def setUp(self):

		self.realGetCommonsMembers = mnislib.getCommonsMembers
		mnislib.getCommonsMembers = mockGetCommonsMembers

	def tearDown(self):

		mnislib.getCommonsMembers = self.realGetCommonsMembers

	def testGetCurrentCommonsMembers(self):

		d = datetime.date.today()
		ds = '{0}-{1}-{2}'.format(d.year, d.month, d.day)
		urlParameters = 'commonsmemberbetween={0}and{1}'.format(ds, ds)
		outputParameters = ['Constituencies', 'Parties', 'HouseMemberships']

		# Test with the default output parameters
		up, op = mnislib.getCurrentCommonsMembers()
		self.assertEqual(up, urlParameters)
		self.assertEqual(op, outputParameters)

		# Test with user defined output parameters
		up, op = mnislib.getCurrentCommonsMembers(['Parameter'])
		self.assertEqual(up, urlParameters)
		self.assertEqual(op, ['Parameter'])


class testGetCommonsMembersOn(unittest.TestCase):

	"""
	Tests getCommonsMembersOn by mocking getCommonsMembers
	and checking that the parameters passed to it are correct.
	"""

	def setUp(self):

		self.realGetCommonsMembers = mnislib.getCommonsMembers
		mnislib.getCommonsMembers = mockGetCommonsMembers


	def tearDown(self):

		mnislib.getCommonsMembers = self.realGetCommonsMembers


	def testGetCommonsMembersOn(self):

		d = datetime.date.today()
		ds = '{0}-{1}-{2}'.format(d.year, d.month, d.day)
		urlParameters = 'commonsmemberbetween={0}and{1}'.format(ds, ds)
		outputParameters = ['Constituencies', 'Parties', 'HouseMemberships']

		# Test with the default output parameters
		up, op = mnislib.getCommonsMembersOn(d)
		self.assertEqual(up, urlParameters)
		self.assertEqual(op, outputParameters)

		# Test with user defined output parameters
		up, op = mnislib.getCommonsMembersOn(d, ['Parameter'])
		self.assertEqual(up, urlParameters)
		self.assertEqual(op, ['Parameter'])


class testGetCommonsMembersBetween(unittest.TestCase):

	"""
	Tests getCommonsMembersBetween by mocking getCommonsMembers
	and checking that the parameters passed to it are correct.
	"""

	def setUp(self):

		self.realGetCommonsMembers = mnislib.getCommonsMembers
		mnislib.getCommonsMembers = mockGetCommonsMembers


	def tearDown(self):

		mnislib.getCommonsMembers = self.realGetCommonsMembers


	def testGetCommonsMembersBetween(self):

		ed = datetime.date.today()
		eds = '{0}-{1}-{2}'.format(ed.year, ed.month, ed.day)
		sd = ed - datetime.timedelta(days=1)
		sds = '{0}-{1}-{2}'.format(sd.year, sd.month, sd.day)
		urlParameters = 'commonsmemberbetween={0}and{1}'.format(sds, eds)
		outputParameters = ['Constituencies', 'Parties', 'HouseMemberships']
		
		# Test with the default output parameters
		up, op = mnislib.getCommonsMembersBetween(sd, ed)
		self.assertEqual(up, urlParameters)
		self.assertEqual(op, outputParameters)

		# Test with user defined output parameters
		up, op = mnislib.getCommonsMembersBetween(sd, ed, ['Parameter'])
		self.assertEqual(up, urlParameters)
		self.assertEqual(op, ['Parameter'])


class testGetCommonsMembersAtElection(unittest.TestCase):

	"""
	Tests getCommonsMembersAtElection by mocking getCommonsMembers
	and checking that the parameters passed to it are correct.
	"""

	def setUp(self):

		self.realGetCommonsMembers = mnislib.getCommonsMembers
		mnislib.getCommonsMembers = mockGetCommonsMembers


	def tearDown(self):

		mnislib.getCommonsMembers = self.realGetCommonsMembers


	def testGetCommonsMembersAtElection(self):

		electionIds = [ \
			'1983', '1987', '1992', '1997', '2001', '2005', '2010', '2015']
		
		outputParameters = ['Constituencies', 'Parties', 'HouseMemberships']

		for electionId in electionIds:

			urlParameters = 'returnedatelection={0}' \
				'%20General%20Election'.format(electionId)

			# Test with the default output parameters
			up, op = mnislib.getCommonsMembersAtElection(electionId)
			self.assertEqual(up, urlParameters)
			self.assertEqual(op, outputParameters)

			# Test with user defined output parameters
			up, op = mnislib.getCommonsMembersAtElection( \
				electionId, ['Parameter'])
			self.assertEqual(up, urlParameters)
			self.assertEqual(op, ['Parameter'])


	def testCommonsMembersAtElectionFails(self):

		self.assertRaises(mnislib.ElectionIdError, \
			mnislib.getCommonsMembersAtElection, '1979')


class testBuildMnisUrl(unittest.TestCase):

	"""
	Tests buildMnisUrl and checks it buildMnisUrl the correct MNIS URL for 
	various combinations of parameters. The parameters are based on those 
	used by the various functions that get Commons members.
	"""

	def testBuildMnisUrlCommonsMemberBetween(self):

		# Test commonsmemberbetween url parameter
		urlParameters = 'commonsmemberbetween=2010-05-06and2015-05-07'
		outputParameters = ['ParameterA', 'ParameterB', 'ParameterC']
		
		correctUrl = 'http://data.parliament.uk/membersdataplatform/' \
			'services/mnis/members/query/House=Commons|Membership=all|' \
			'commonsmemberbetween=2010-05-06and2015-05-07/ParameterA|' \
			'ParameterB|ParameterC'

		returnedUrl = mnislib.buildMnisUrl(urlParameters, outputParameters)
		self.assertEqual(returnedUrl, correctUrl)


	def testBuildMnisUrlReturnedAtElection(self):

		# Test returnedatelection url parameter
		urlParameters = 'returnedatelection=2001%20General%20Election'
		outputParameters = ['ParameterA', 'ParameterB', 'ParameterC']
		
		correctUrl = 'http://data.parliament.uk/membersdataplatform/' \
			'services/mnis/members/query/House=Commons|Membership=all|' \
			'returnedatelection=2001%20General%20Election/ParameterA|' \
			'ParameterB|ParameterC'

		returnedUrl = mnislib.buildMnisUrl(urlParameters, outputParameters)
		self.assertEqual(returnedUrl, correctUrl)


	def testBuildMnisUrlFails(self):

		# Test buildMnisUrl fails with more than three output parameters
		self.assertRaises(mnislib.ParameterError, mnislib.buildMnisUrl, '', \
			['ParameterA', 'ParameterB', 'ParameterC', 'ParameterD'])


class testGetCommonsMembers(unittest.TestCase):

	"""
	Tests getCommonsMebers to check it properly downloads data from the MNIS
	API. The returned members are stored in a global variable and are used
	in subsequent tests to check functions which handle the returned data.
	"""

	def testGetCommonsMembers(self):
		
		# Get Commons members at GE2015		
		members = mnislib.getCommonsMembers( \
			'commonsmemberbetween=2015-05-07and2015-05-07', \
			['Constituencies', 'Parties', 'HouseMemberships'])

		# Sort members to ensure they appear in list name alphabetical order
		members.sort(key=lambda member: member['ListAs'])

		# Test the downloaded data is correct
		self.assertEqual(members, correctMembers)


class testGetIdForMember(unittest.TestCase):

	"""Tests getIdForMember and checks it returns the correct id."""

	def testGetIdForMember(self):

		# Check first member listed alphabetically: Diane Abbott
		g = mnislib.getIdForMember(correctMembers[0])
		self.assertEqual(g, '172')

		# Check last member listed alphabetically: Daniel Zeichner
		g = mnislib.getIdForMember(correctMembers[649])
		self.assertEqual(g, '4382')


class testGetListNameForMember(unittest.TestCase):

	"""Tests getListNameForMember and checks it returns the correct name."""

	def testGetListNameForMember(self):

		# Check first member listed alphabetically: Diane Abbott
		ln = mnislib.getListNameForMember(correctMembers[0])
		self.assertEqual(ln, 'Abbott, Ms Diane')

		# Check last member listed alphabetically: Daniel Zeichner
		ln = mnislib.getListNameForMember(correctMembers[649])
		self.assertEqual(ln, 'Zeichner, Daniel')


class testGetGenderForMember(unittest.TestCase):

	"""Tests getGenderForMember and checks it returns the correct gender."""

	def testGetGenderForMember(self):

		# Check first member listed alphabetically: Diane Abbott
		g = mnislib.getGenderForMember(correctMembers[0])
		self.assertEqual(g, 'F')

		# Check last member listed alphabetically: Daniel Zeichner
		g = mnislib.getGenderForMember(correctMembers[649])
		self.assertEqual(g, 'M')


class testGetDateOfBirthForMember(unittest.TestCase):

	"""Tests getDateOfBirthForMember and checks it returns the correct DoB."""

	def testGetDateOfBirthForMember(self):

		# Check first member listed alphabetically: Diane Abbott
		dob = mnislib.getDateOfBirthForMember(correctMembers[0])
		d = datetime.date(1953, 9, 27)
		self.assertEqual(dob, d)

		# Check last member listed alphabetically: Daniel Zeichner
		dob = mnislib.getDateOfBirthForMember(correctMembers[649])
		d = datetime.date(1956, 11, 9)
		self.assertEqual(dob, d)


class testGetConstituencyForMember(unittest.TestCase):

	"""Tests getConstituencyForMember and checks it returns the correct one."""

	def testGetConstituencyForMember(self):

		# Set onDate to GE2015 for the latest constituency in the test data
		d = datetime.date(2015, 5, 7)

		# Check first member listed alphabetically: Diane Abbott
		con = mnislib.getConstituencyForMember(correctMembers[0], d)
		self.assertEqual(con, 'Hackney North and Stoke Newington')

		# Check last member listed alphabetically: Daniel Zeichner
		con = mnislib.getConstituencyForMember(correctMembers[649], d)
		self.assertEqual(con, 'Cambridge')

		# Check Boris Johnson for constituency as at GE2015
		con = mnislib.getConstituencyForMember(correctMembers[319], d)
		self.assertEqual(con, 'Uxbridge and South Ruislip')

		# Check Boris Johnson for constituency as at GE2001
		d = datetime.date(2001, 6, 7)
		con = mnislib.getConstituencyForMember(correctMembers[319], d)
		self.assertEqual(con, 'Henley')


class testGetPartyForMember(unittest.TestCase):

	"""Tests getPartyForMember and checks it returns the correct one."""

	def testGetPartyForMember(self):

		# Set onDate to GE2015 for the latest party in the test data
		d = datetime.date(2015, 5, 7)

		# Check first member listed alphabetically: Diane Abbott
		p = mnislib.getPartyForMember(correctMembers[0], d)
		self.assertEqual(p, 'Labour')

		# Check last member listed alphabetically: Daniel Zeichner
		p = mnislib.getPartyForMember(correctMembers[649], d)
		self.assertEqual(p, 'Labour')

		# Check Douglas Carswell for party as at GE2015
		p = mnislib.getPartyForMember(correctMembers[91], d)
		self.assertEqual(p, 'UK Independence Party')

		# Check Douglas Carswell for party as at GE2010
		d = datetime.date(2010, 5, 6)
		p = mnislib.getPartyForMember(correctMembers[91], d)
		self.assertEqual(p, 'Conservative')


class testIsDateInMembership(unittest.TestCase):

	"""Tests isDateInMembership and checks it returns the correct booleans."""

	def testOpenMembership(self):

		# Mock open membership
		openMembershipJson = '{"EndDate": {"@xmlns:xsi": ' + \
			'"http://www.w3.org/2001/XMLSchema-instance", ' + \
			'"@xsi:nil": "true"}, "StartDate": "2015-05-07T00:00:00"}'

		openMembership = json.loads(openMembershipJson)

		# Check date at start of membership
		self.assertTrue(mnislib.isDateInMembership( \
			openMembership, datetime.date(2015, 5, 7)))

		# Check date inside membership
		self.assertTrue(mnislib.isDateInMembership( \
			openMembership, datetime.date(2016, 1, 1)))

		# Check current date: should be true
		self.assertTrue(mnislib.isDateInMembership( \
			openMembership, datetime.date.today()))

		# Check date before membership
		self.assertFalse(mnislib.isDateInMembership( \
			openMembership, datetime.date(2015, 5, 6)))

		# Check date after membership
		self.assertFalse(mnislib.isDateInMembership( \
			openMembership, datetime.date(2100, 1, 1)))


	def testClosedMembership(self):

		# Mock closed membership
		closedMembershipJson = '{"EndDate": "2015-03-30T00:00:00", ' + \
			'"StartDate": "2010-05-06T00:00:00"}'

		closedMembership = json.loads(closedMembershipJson)

		# Check date at start of membership
		self.assertTrue(mnislib.isDateInMembership( \
			closedMembership, datetime.date(2010, 5, 6)))

		# Check date inside membership
		self.assertTrue(mnislib.isDateInMembership( \
			closedMembership, datetime.date(2013, 1, 1)))

		# Check date at end of membership
		self.assertTrue(mnislib.isDateInMembership( \
			closedMembership, datetime.date(2015, 3, 30)))

		# Check date before membership
		self.assertFalse(mnislib.isDateInMembership( \
			closedMembership, datetime.date(2010, 5, 5)))

		# Check date after membership
		self.assertFalse(mnislib.isDateInMembership( \
			closedMembership, datetime.date(2015, 3, 31)))


class testGetServiceDataForMember(unittest.TestCase):

	"""Tests getServiceDataForMember and checks it returns correct data."""

	def testGetServiceDataForMember(self):

		# Set onDate to GE2015 to match the test data
		d = datetime.date(2015, 5, 7)

		# Check first member listed alphabetically: Diane Abbott
		sd = mnislib.getServiceDataForMember(correctMembers[0], d)
		self.assertEqual(sd, (datetime.date(1987, 6, 11), 10035))

		# Check last member listed alphabetically: Daniel Zeichner
		sd = mnislib.getServiceDataForMember(correctMembers[649], d)
		self.assertEqual(sd, (datetime.date(2015, 5, 7), 0))

		# Check Boris Johnson as member with interrupted membership
		sd = mnislib.getServiceDataForMember(correctMembers[319], d)
		self.assertEqual(sd, (datetime.date(2001, 6, 7), 2530))


class testGetMembershipDays(unittest.TestCase):

	"""Tests getMembershipDays and checks it returns correct counts."""

	def testGetMembershipDays(self):

		# Mock a membership that spans a number of elections
		membershipJson = '{"EndDate": "2015-03-30T00:00:00", ' + \
			'"StartDate": "1987-06-11T00:00:00"}'

		membership = json.loads(membershipJson)

		# Test with an onDate after the end of the membership
		d = datetime.date.today()
		self.assertEqual(mnislib.getMembershipDays(membership, d), 10035)

		# Test with an onDate halfway through the membership
		d = datetime.date(1997, 4, 8)
		self.assertEqual(mnislib.getMembershipDays(membership, d), 3565)	

		# Test with an onDate before the membership
		d = datetime.date(1987, 6, 10)
		self.assertEqual(mnislib.getMembershipDays(membership, d), 0)


	def testGetMembershipDaysFails(self):

		# Mock a membership with invalid data
		membershipJson = '{"EndDate": "2015-03-30T00:00:00", ' + \
			'"StartDate": "2015-05-07T00:00:00"}'

		membership = json.loads(membershipJson)

		# Test with an onDate after the end of the membership
		d = datetime.date.today()

		self.assertRaises(mnislib.MembershipError, \
			mnislib.getMembershipDays, membership, d)


class testGetSummaryDataForMembers(unittest.TestCase):

	"""Tests getSummaryDataForMembers and checks it returns correct data."""

	def testGetSummaryDataForMembers(self):

		# Set onDate to GE2015 to match the test data
		d = datetime.date(2015, 5, 7)

		# Get summary data
		sd = mnislib.getSummaryDataForMembers(correctMembers, d)

		# Check first member listed alphabetically: Diane Abbott
		self.assertEqual(sd[0]['member_id'], '172')
		self.assertEqual(sd[0]['list_name'], 'Abbott, Ms Diane')
		self.assertEqual(sd[0]['constituency'], \
			'Hackney North and Stoke Newington')
		self.assertEqual(sd[0]['party'], 'Labour')
		self.assertEqual(sd[0]['date_of_birth'], datetime.date(1953, 9, 27))
		self.assertEqual(sd[0]['gender'], 'F')
		self.assertEqual(sd[0]['first_start_date'], datetime.date(1987, 6, 11))
		self.assertEqual(sd[0]['days_service'], 10035)

		# Check last member listed alphabetically: Daniel Zeichner
		self.assertEqual(sd[649]['member_id'], '4382')
		self.assertEqual(sd[649]['list_name'], 'Zeichner, Daniel')
		self.assertEqual(sd[649]['constituency'], 'Cambridge')
		self.assertEqual(sd[649]['party'], 'Labour')
		self.assertEqual(sd[649]['date_of_birth'], datetime.date(1956, 11, 9))
		self.assertEqual(sd[649]['gender'], 'M')
		self.assertEqual(sd[649]['first_start_date'], datetime.date(2015, 5, 7))
		self.assertEqual(sd[649]['days_service'], 0)


class testSaveSummaryDataForMembers(unittest.TestCase):

	"""Tests saveSummaryDataForMembers and checks it writes correct data."""

	def setUp(self):

		# Set filename for testing
		self.filename = "unittest.csv"

		# Set onDate to GE2015 to match the test data
		d = datetime.date(2015, 5, 7)

		# getthe summaryData for the members
		sd = mnislib.getSummaryDataForMembers(correctMembers, d)

		# Write member data to disk as csv
		mnislib.saveSummaryDataForMembers(sd, self.filename)

	def tearDown(self):

		if os.path.exists(self.filename):
			os.remove(self.filename)

	def testSaveSummaryDataForMembers(self):

		# Test if the file has been written to disk
		self.assertTrue(os.path.exists(self.filename))

		# Try reading the file in as a csv
		with open(self.filename) as csvFile:
			reader = csv.reader(csvFile)
			rows = list(reader)

			header = [ \
				'member_id', \
				'list_name', \
				'constituency', \
				'party', \
				'date_of_birth', \
				'gender', \
				'first_start_date', \
				'days_service'\
			]

			self.assertEqual(rows[0], header)

			firstMember = [ \
				'172', \
				'Abbott, Ms Diane', \
				'Hackney North and Stoke Newington', \
				'Labour', \
				'1953-09-27', \
				'F', \
				'1987-06-11', \
				'10035' \
			]

			self.assertEqual(rows[1], firstMember)

			lastMember = [ \
				'4382', \
				'Zeichner, Daniel', \
				'Cambridge', \
				'Labour', \
				'1956-11-09', \
				'M', \
				'2015-05-07', \
				'0' \
			]

			self.assertEqual(rows[650], lastMember)


class testDownloadMembers(unittest.TestCase):

	"""
	Tests downloadMembers and checks it writes correct data. 

	For now this test is empty as the function is a very simple wrapper around 
	getCommonsMembersOn(), getSummaryDataForMembers(), 
	and saveSummaryDataForMembers(), each of which are tested elsewhere. 
	Retesting the combined functionality here would only add another download 
	to the test suite and unduly increase the time it takes to run the tests. 
	If this function becomes more complex in future a test should be added.
	"""

	def testDownloadMembers(self):

		pass


class testConvertMnisDatetime(unittest.TestCase):

	"""Tests convertMnisDatetime and checks it converts dates correctly."""

	def testConvertMnisDatetime(self):

		exampleDatetimes = [ \
			'1992-04-09T00:00:00', \
			'1997-05-01T00:00:00', \
			'2001-06-07T00:00:00', \
			'2005-05-05T00:00:00', \
			'2010-05-06T00:00:00', \
			'2015-05-07T00:00:00', \
		]

		convertedDatetimes = [ \
			datetime.date(1992, 4, 9), \
			datetime.date(1997, 5, 1), \
			datetime.date(2001, 6, 7), \
			datetime.date(2005, 5, 5), \
			datetime.date(2010, 5, 6), \
			datetime.date(2015, 5, 7), \
		]

		for i in range(len(exampleDatetimes)):

			d = mnislib.convertMnisDatetime(exampleDatetimes[i])
			self.assertEqual(d, convertedDatetimes[i])


class testIsDateInRange(unittest.TestCase):

	"""Tests isDateInRange and checks it returns the correct boolean."""

	def testIsDateInRange(self):

		dates = [ \
			datetime.date(2015, 5, 6), \
			datetime.date(2015, 5, 7), \
			datetime.date(2015, 5, 8), \
		]

		# Test where onDate, startDate and endDate are equal
		self.assertTrue(mnislib.isDateInRange(dates[0], dates[0], dates[0]))

		# Test where onDate equals startDate, and endDate is later
		self.assertTrue(mnislib.isDateInRange(dates[0], dates[0], dates[1]))

		# Test where onDate equals endDate, and startDate is earlier
		self.assertTrue(mnislib.isDateInRange(dates[1], dates[0], dates[1]))

		# Test where onDate is after startDate and before endDate
		self.assertTrue(mnislib.isDateInRange(dates[1], dates[0], dates[2]))

		# Test where onDate is befpre startDate and endDate
		self.assertFalse(mnislib.isDateInRange(dates[0], dates[1], dates[2]))

		# Test where onDate is after startDate and endDate
		self.assertFalse(mnislib.isDateInRange(dates[2], dates[0], dates[1]))



if __name__ == '__main__':
	unittest.main()